@@ -13,6 +13,8 @@ using System.Diagnostics;
 
 using Beagle.Util;
 using Beagle.Daemon;
+using Beagle.Util.Xmp;
+using SemWeb;
 
 namespace Beagle.Filters {
 
@@ -21,6 +23,7 @@ namespace Beagle.Filters {
 		public FilterPdf ()
 		{
 			SnippetMode = true;
+			SetFileType ("document");
 		}
 
 		protected override void RegisterSupportedTypes ()
@@ -38,7 +41,7 @@ namespace Beagle.Filters {
 		{
 			// create new external process
 			pc = new SafeProcess ();
-			pc.Arguments = new string [] { "pdfinfo", FileInfo.FullName };
+			pc.Arguments = new string [] { "pdfinfo", "-meta", FileInfo.FullName };
 			pc.RedirectStandardOutput = true;
 			// See FIXME below for why this is false.
 			pc.RedirectStandardError = false;
@@ -47,6 +50,7 @@ namespace Beagle.Filters {
 			// use more than 100 megs memory.
 			pc.CpuLimit = 90;
 			pc.MemLimit = 100*1024*1024;
+			pc.UseLangC = true;
 
 			try {
 				pc.Start ();
@@ -92,6 +96,11 @@ namespace Beagle.Filters {
 					case "Producer":
 						strMetaTag = "dc:appname";
 						break;
+					case "Metadata":
+						string xmpString = pout.ReadToEnd();
+						XmpFile xmp = new XmpFile (new MemoryStream(System.Text.Encoding.ASCII.GetBytes(xmpString)));
+						AddXmpProperties(xmp);
+						break;
 					}
 					if (strMetaTag != null) {
 						if (bKeyword)
@@ -110,7 +119,7 @@ namespace Beagle.Filters {
 			// Log any errors or warnings from stderr
 			pout = new StreamReader (pc.StandardError);
 			while ((str = pout.ReadLine ()) != null)
-				Log.Warn ("pdfinfo [{0}]: {1}", Uri, str);
+				Log.Warn ("pdfinfo [{0}]: {1}", Indexable.Uri, str);
 
 			pout.Close ();
 #endif
@@ -202,6 +211,49 @@ namespace Beagle.Filters {
 #endif
 			pc.Close ();
 		}
+		
+		private void AddXmpProperties (XmpFile xmp)
+		{
+			Resource subject_anon = null;
+			Resource creator_anon = null;
+			Resource rights_anon = null;
+			Resource title_anon = null;
+
+			foreach (Statement stmt in xmp.Store) {
+				if (stmt.Predicate == MetadataStore.Namespaces.Resolve ("dc:subject")) {
+					//Console.WriteLine ("found subject");
+					subject_anon = stmt.Object;
+				} else if (stmt.Predicate == MetadataStore.Namespaces.Resolve ("dc:creator")) {
+					//Console.WriteLine ("found creator");
+					creator_anon = stmt.Object;
+				} else if (stmt.Predicate == MetadataStore.Namespaces.Resolve ("dc:rights")) {
+					rights_anon = stmt.Object;
+				} else if (stmt.Predicate == MetadataStore.Namespaces.Resolve ("dc:title")) {
+					if (stmt.Object is Literal)
+						AddProperty (Beagle.Property.New ("dc:title", ((Literal)stmt.Object).Value));
+					else if (stmt.Object is BNode)
+						title_anon = stmt.Object;
+				} else if (stmt.Predicate == MetadataStore.Namespaces.Resolve ("cc:license")) {
+					AddProperty (Beagle.Property.NewKeyword ("fixme:license", ((Literal)stmt.Object).Value));
+				}
+			}
+			
+			foreach (Statement stmt in xmp.Store) {
+				if (stmt.Subject == subject_anon && 
+				    stmt.Predicate != MetadataStore.Namespaces.Resolve ("rdf:type")) {
+					AddProperty (Beagle.Property.New ("dc:subject", ((Literal)stmt.Object).Value));
+				} else if (stmt.Subject == creator_anon &&  
+					   stmt.Predicate != MetadataStore.Namespaces.Resolve ("rdf:type")) {
+					AddProperty (Beagle.Property.New ("dc:creator", ((Literal)stmt.Object).Value));
+				} else if (stmt.Subject == rights_anon &&  
+					   stmt.Predicate != MetadataStore.Namespaces.Resolve ("rdf:type")) {
+					AddProperty (Beagle.Property.New ("dc:rights", ((Literal)stmt.Object).Value));
+				} else if (stmt.Subject == title_anon &&
+					   stmt.Predicate != MetadataStore.Namespaces.Resolve ("rdf:type")) {
+					AddProperty (Beagle.Property.New ("dc:title", ((Literal)stmt.Object).Value));
+				}
+			}
+		}
 	}
 }
 
