@@ -39,6 +39,7 @@ using Log = Beagle.Util.Log;
 using Stopwatch = Beagle.Util.Stopwatch;
 
 namespace Beagle.Daemon {
+
 	class BeagleDaemon {
 
 		public static Thread MainLoopThread = null;
@@ -58,12 +59,16 @@ namespace Beagle.Daemon {
 			set { disable_textcache = value; }
 		}
 
+#if ENABLE_AVAHI
+		private static Beagle.Daemon.Network.Zeroconf zeroconf = null;
+#endif
+
 		public static bool StartServer ()
 		{
 			Logger.Log.Debug ("Starting messaging server");
 
 			try {
-				server = new Server ("socket");
+				server = new Server ("socket", false, Conf.Networking.GetOption (Conf.Names.ServiceEnabled, false));
 				server.Start ();
 			} catch (InvalidOperationException) {
 				return false;
@@ -111,7 +116,7 @@ namespace Beagle.Daemon {
 
 				if (send_sigprof) {
 					Log.Debug ("Suspicious memory size change detected.  Sending SIGPROF to ourself ({0})", sigprof_count++);
-					Mono.Unix.Native.Syscall.kill (Process.GetCurrentProcess ().Id, Mono.Unix.Native.Signum.SIGPROF);
+					Mono.Unix.Native.Syscall.kill (System.Diagnostics.Process.GetCurrentProcess ().Id, Mono.Unix.Native.Signum.SIGPROF);
 				}
 			}
 		}
@@ -137,65 +142,62 @@ namespace Beagle.Daemon {
 
 		private static void PrintUsage ()
 		{
-			string usage =
-				"beagled: The daemon to the Beagle search system.\n" +
-				"Web page: http://beagle-project.org\n" +
-				"Copyright (C) 2004-2006 Novell, Inc.\n\n";
+			VersionFu.PrintHeader ();
 
-			usage +=
+			string usage =
 				"Usage: beagled [OPTIONS]\n\n" +
 				"Options:\n" +
 				"  --foreground, --fg\tRun the daemon in the foreground.\n" +
 				"  --background, --bg\tRun the daemon in the background.\n" +
-				"  --replace\t\tReplace a running daemon with a new instance.\n" +
-				"  --debug\t\tWrite out debugging information.\n" +
-				"  --debug-memory\tWrite out debugging information about memory use.\n" +
-				"  --indexing-test-mode\tRun in foreground, and exit when fully indexed.\n" +
-				"  --indexing-delay\tTime to wait before indexing.  (Default 60 seconds)\n" +
 				"  --backend\t\tConfigure which backends to use.  Specifically:\n" +
 				"    --backend <name>\tOnly start backend 'name'\n" +
 				"    --backend +<name>\tAdditionally start backend 'name'\n" +
 				"    --backend -<name>\tDisable backend 'name'\n" +
-				"  --allow-backend\t(DEPRECATED) Start only the specific backend.\n" +
-				"  --deny-backend\t(DEPRECATED) Deny a specific backend.\n" +
+				"  --replace\t\tReplace a running daemon with a new instance.\n" +
 				"  --list-backends\tList all the available backends.\n" +
 				"  --add-static-backend\tAdd a static backend by path.\n" + 
+				"  --help\t\tPrint this usage message.\n" +
+				"  --version\t\tPrint version information.\n" +
+				"\n" +
+				"Advanced options:\n" +
+				"  --debug\t\tWrite out debugging information.\n" +
+				"  --debug-memory\tWrite out debugging information about memory use.\n" +
+				"  --indexing-test-mode\tRun in foreground, and exit when fully indexed.\n" +
+				"  --indexing-delay <t>\tWait 't' seconds before indexing.  (Default 60 seconds)\n" +
 				"  --disable-scheduler\tDisable the use of the scheduler.\n" +
-				// FIXME: Expose this to the user ?
-				//"  --disable-text-cache\tDisable the use of the text cache used to provide snippets
-				"  --help\t\tPrint this usage message.\n";
+				"  --disable-text-cache\tDisable the use of the text cache used to provide snippets.\n";
 
 			Console.WriteLine (usage);
 		}
 
 		public static bool StartupProcess ()
 		{
-			Log.Debug ("Beginning main loop");
-
 			// Profile our initialization
 			Stopwatch stopwatch = new Stopwatch ();
 			stopwatch.Start ();
 
 			// Fire up our server
 			if (! StartServer ()) {
-				if (arg_replace)
-				{
-					ReplaceExisting ();
-				}		
-				else {
+				if (! arg_replace) {
 					Logger.Log.Error ("Could not set up the listener for beagle requests.  "
 							  + "There is probably another beagled instance running.  "
 							  + "Use --replace to replace the running service");
 					Environment.Exit (1);
 				}
+
+				ReplaceExisting ();
 			}
 			
 			// Set up out-of-process indexing
 			LuceneQueryable.IndexerHook = new LuceneQueryable.IndexerCreator (RemoteIndexer.NewRemoteIndexer);
 
+			Config config = Conf.Get (Conf.Names.DaemonConfig);
+
 			// Initialize synchronization to keep the indexes local if PathFinder.StorageDir
 			// is on a non-block device, or if BEAGLE_SYNCHRONIZE_LOCALLY is set
-			if ((! SystemInformation.IsPathOnBlockDevice (PathFinder.StorageDir) && Conf.Daemon.IndexSynchronization) ||
+
+			if ((! SystemInformation.IsPathOnBlockDevice (PathFinder.StorageDir) &&
+			     config.GetOption (Conf.Names.IndexSynchronization, true)) ||
 			    Environment.GetEnvironmentVariable ("BEAGLE_SYNCHRONIZE_LOCALLY") != null)
 				IndexSynchronization.Initialize ();
 
@@ -203,24 +205,22 @@ namespace Beagle.Daemon {
 			Logger.Log.Debug ("Starting QueryDriver");
 			QueryDriver.Start ();
 
-			bool initially_on_battery = SystemInformation.UsingBattery && ! Conf.Indexing.IndexOnBattery;
+			// Start our battery monitor so we can shut down the
+			// scheduler if needed.
+			BatteryMonitor.Init ();
+
+			bool initially_on_battery = ! BatteryMonitor.UsingAC && ! config.GetOption (Conf.Names.IndexOnBattery, false);
 
 			// Start the Global Scheduler thread
 			if (! arg_disable_scheduler) {
 				if (! initially_on_battery) {
 					Logger.Log.Debug ("Starting Scheduler thread");
 					Scheduler.Global.Start ();
-				} else
+				} else {
 					Log.Debug ("Beagle started on battery, not starting scheduler thread");
+				}
 			}
 
-			// Poll the battery status so we can shut down the
-			// scheduler if needed.  Ideally at some point this
-			// will become some sort of D-BUS signal, probably from
-			// something like gnome-power-manager.
-			prev_on_battery = initially_on_battery;
-			GLib.Timeout.Add (5000, CheckBatteryStatus);
-
 			// Start our Inotify threads
 			Inotify.Start ();
 	
@@ -228,6 +228,10 @@ namespace Beagle.Daemon {
 			// warning if not.  The actual advice calls will fail silently.
 			FileAdvise.TestAdvise ();
 
+#if ENABLE_AVAHI
+               	        zeroconf = new Beagle.Daemon.Network.Zeroconf ();
+#endif
+
 			Conf.WatchForUpdates ();
 
 			stopwatch.Stop ();
@@ -242,6 +246,7 @@ namespace Beagle.Daemon {
 				Scheduler.Global.EmptyQueueEvent += OnEmptySchedulerQueue;
 				Scheduler.Global.Add (null); // pulse the scheduler
 			}
+
 			return false;
 		}
 
@@ -366,24 +371,6 @@ namespace Beagle.Daemon {
 					++i; // we used next_arg
 					break;
 				
-				case "--allow-backend":
-					// FIXME: This option is deprecated and will be removed in a future release.
-					// --allow-backend is deprecated, use --backends 'name' instead
-					// it will disable reading the list of enabled/disabled backends
-					// from conf and start the backend given
-					if (next_arg != null)
-						QueryDriver.OnlyAllow (next_arg);
-					++i; // we used next_arg
-					break;
-					
-				case "--deny-backend":
-					// FIXME: This option is deprecated and will be removed in a future release.
-					// deprecated: use --backends -'name' instead
-					if (next_arg != null)
-						QueryDriver.Deny (next_arg);
-					++i; // we used next_arg
-					break;
-
 			       case "--add-static-backend": 
 					if (next_arg != null)
 						QueryDriver.AddStaticQueryable (next_arg);
@@ -414,6 +401,11 @@ namespace Beagle.Daemon {
 				case "--disable-text-cache":
 					disable_textcache = true;
 					break;
+					
+				case "--version":
+					VersionFu.PrintVersion ();
+					Environment.Exit (0);
+					break;
 
 				default:
 					Console.WriteLine ("Unknown argument '{0}'", arg);
@@ -423,10 +415,14 @@ namespace Beagle.Daemon {
 				}
 			}
 
+			if (Environment.GetEnvironmentVariable ("SABAYON_SESSION_RUNNING") == "yes") {
+				Console.WriteLine ("Beagle is running underneath Sabayon, exiting.");
+				Environment.Exit (0);
+			}
+
 			if (arg_indexing_test_mode) {
 				LuceneQueryable.OptimizeRightAway = true;
 			}
-				
 
 			// Bail out if we are trying to run as root
 			if (Environment.UserName == "root" && Environment.GetEnvironmentVariable ("SUDO_USER") != null) {
@@ -437,7 +433,7 @@ namespace Beagle.Daemon {
 				Environment.Exit (-1);
 			}
 
-			if (Environment.UserName == "root" && ! Conf.Daemon.AllowRoot) {
+			if (Environment.UserName == "root" && ! Conf.Daemon.GetOption (Conf.Names.AllowRoot, false)) {
 				Console.WriteLine ("You can not run beagle as root.  Beagle is designed to run from your own");
 				Console.WriteLine ("user account.  If you want to create multiuser or system-wide indexes, use");
 				Console.WriteLine ("the beagle-build-index tool.");
@@ -455,22 +451,18 @@ namespace Beagle.Daemon {
 
 			MainLoopThread = Thread.CurrentThread;
 
-			Log.Initialize (PathFinder.LogDir,
-					"Beagle", 
-					// FIXME: We always turn on full debugging output!  We are still
-					// debugging this code, after all...
-					//arg_debug ? LogLevel.Debug : LogLevel.Warn,
-					LogLevel.Debug,
-					arg_fg);
-
+			// FIXME: We always turn on full debugging output!  We are still
+			// debugging this code, after all...
+			// arg_debug ? LogLevel.Debug : LogLevel.Warn
+			
+			Log.Initialize (PathFinder.LogDir, "Beagle", LogLevel.Debug, arg_fg);
 			Log.Always ("Starting Beagle Daemon (version {0})", ExternalStringsHack.Version);
 			Log.Always ("Running on {0}", SystemInformation.MonoRuntimeVersion);
-			Log.Always ("Using sqlite version {0}", ExternalStringsHack.SqliteVersion);
 			Log.Always ("Command Line: {0}",
 					   Environment.CommandLine != null ? Environment.CommandLine : "(null)");
 
 			if (! ExtendedAttribute.Supported) {
-				Logger.Log.Warn ("Extended attributes are not supported on this filesystem.  " +
+				Logger.Log.Warn ("Extended attributes are not supported on this filesystem. " +
 						 "Performance will suffer as a result.");
 			}
 
@@ -479,9 +471,16 @@ namespace Beagle.Daemon {
 				Log.Warn ("*** Snippets will not be returned for documents indexed in this session.");
 			}
 
+			// Check if global configuration files are installed
+			if (! Conf.CheckGlobalConfig ()) {
+				Console.WriteLine ("Global configuration files not found in '{0}'", PathFinder.ConfigDataDir);
+				Environment.Exit (-1);
+			}
+
 			// Start our memory-logging thread
-			if (arg_debug_memory)
+			if (arg_debug_memory) {
 				ExceptionHandlingThread.Start (new ThreadStart (LogMemoryUsage));
+			}
 
 			// Do BEAGLE_EXERCISE_THE_DOG_HARDER-related processing.
 			ExerciseTheDogHarder ();
@@ -501,19 +500,23 @@ namespace Beagle.Daemon {
 			QueryDriver.Init ();
 			Server.Init ();
 
+#if MONO_1_9
+			Shutdown.SetupSignalHandlers (new Shutdown.SignalHandler (HandleSignal));
+#else
 			SetupSignalHandlers ();
+#endif
+
 			Shutdown.ShutdownEvent += OnShutdown;
 
 			main_loop = new MainLoop ();
 			Shutdown.RegisterMainLoop (main_loop);
-
+			
 			// Defer all actual startup until the main loop is
 			// running.  That way shutdowns during the startup
 			// process work correctly.
 			GLib.Idle.Add (new GLib.IdleHandler (StartupProcess));
 
 			// Start our event loop.
-			Logger.Log.Debug ("Starting main loop");
 			main_loop.Run ();
 
 			// We're out of the main loop now, join all the
@@ -531,27 +534,6 @@ namespace Beagle.Daemon {
 			Log.Always ("Beagle daemon process shut down cleanly.");
 		}
 
-		/////////////////////////////////////////////////////////////////////////////
-		
-		private static bool prev_on_battery = false;
-
-		private static bool CheckBatteryStatus ()
-		{
-			if (prev_on_battery && (! SystemInformation.UsingBattery || Conf.Indexing.IndexOnBattery)) {
-				if (! SystemInformation.UsingBattery)
-					Log.Info ("Deletected a switch from battery to AC power.  Restarting scheduler.");
-				Scheduler.Global.Start ();
-				prev_on_battery = false;
-			} else if (! prev_on_battery && SystemInformation.UsingBattery && ! Conf.Indexing.IndexOnBattery) {
-				Log.Info ("Detected a switch from AC power to battery.  Stopping scheduler.");
-				Scheduler.Global.Stop ();
-				prev_on_battery = true;
-			}
-
-			return true;
-		}
-
-
 		/////////////////////////////////////////////////////////////////////////////
 
 		private delegate int XIOErrorHandler (IntPtr display);
@@ -601,7 +583,8 @@ namespace Beagle.Daemon {
 				return;
 
 			// Set shutdown flag to true so that other threads can stop initializing
-			if ((Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR1)
+			if ((Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR1 &&
+			    (Mono.Unix.Native.Signum) signal != Mono.Unix.Native.Signum.SIGUSR2)
 				Shutdown.ShutdownRequested = true;
 
 			// Do all signal handling work in the main loop and not in the signal handler.
@@ -612,17 +595,22 @@ namespace Beagle.Daemon {
 		{
 			Logger.Log.Debug ("Handling signal {0} ({1})", signal, (Mono.Unix.Native.Signum) signal);
 
+			// Pass the signals to the helper too.
+			GLib.Idle.Add (new GLib.IdleHandler (delegate ()
+							    {
+								    RemoteIndexer.SignalRemoteIndexer ((Mono.Unix.Native.Signum) signal);
+								    return false; 
+							    }));
+
 			// If we get SIGUSR1, turn the debugging level up.
 			if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR1) {
 				LogLevel old_level = Log.Level;
 				Log.Level = LogLevel.Debug;
 				Log.Debug ("Moving from log level {0} to Debug", old_level);
-			}
-
-			// Send informational signals to the helper too.
-			if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR1 ||
-			    (Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR2) {
-				GLib.Idle.Add (new GLib.IdleHandler (delegate () { RemoteIndexer.SignalRemoteIndexer ((Mono.Unix.Native.Signum) signal); return false; }));
+				return;
+			} else if ((Mono.Unix.Native.Signum) signal == Mono.Unix.Native.Signum.SIGUSR2) {
+				// Debugging hook for beagled
+				QueryDriver.DebugHook ();
 				return;
 			}
 
@@ -634,6 +622,9 @@ namespace Beagle.Daemon {
 
 		private static void OnShutdown ()
 		{
+#if ENABLE_AVAHI
+			zeroconf.Dispose ();
+#endif			
 			// Stop our Inotify threads
 			Inotify.Stop ();
 
@@ -647,7 +638,6 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////////////////////////////////////////
 
-
 		private static ArrayList exercise_files = new ArrayList ();
 
 		private static void ExerciseTheDogHarder ()
