@@ -43,10 +43,13 @@ class SnippetTestTool {
 		foreach (string arg in args)
 			query.AddText (arg);
 
-		string snippet;
-		snippet = SnippetFu.GetSnippet (args,
-						new SnippetFu.StringSource (Console.ReadLine));
+		SnippetReader snippet_reader;
+		// FIXME: Oops ... does not quit by passing empty line
+		snippet_reader = SnippetFu.GetSnippet (args, Console.In, false, -1, -1); //delegate {
 
-		Console.WriteLine ("Snippet:\n'{0}'", snippet);
+		bool first = true;
+		foreach (SnippetLine snippet_line in snippet_reader.GetSnippet ()) {
+			Console.WriteLine ("\nSnippet:\n{0}", snippet_line.ToString ());
+		}
 	}
 }
